clear
close all

data = readtable('Experimental_results.xlsx');

Cf = [200e-4,680e-4,6.29e-1,57e-1,97e-1]; % Fluid conductivity [S/m]

% Bulk conductivity under wet conditions of three orthogonal directions (x,y,z) [S/m]
C1 = [1./data.Rwet1_x,1./data.Rwet1_y,1./data.Rwet1_z];
C2 = [1./data.Rwet2_x,1./data.Rwet2_y,1./data.Rwet2_z];
C3 = [1./data.Rwet3_x,1./data.Rwet3_y,1./data.Rwet3_z];
C4 = [1./data.Rwet4_x,1./data.Rwet4_y,1./data.Rwet4_z];
C5 = [1./data.Rwet5_x,1./data.Rwet5_y,1./data.Rwet5_z];

Cdry = 1./data.Rdry_mean; % Dry conductivity (mean values of three directions)

Fi = data.Rwet5_mean/(1/Cf(5)); % Initial values of F for the inversion

t = 0.38; % Hittorf number for NaCl
X = [];

options = optimoptions('lsqcurvefit','Display','off');
lb = [0,0]; % Lower bound
ub = []; % Upper bound

% Inversions
Xx = []; Xy = []; Xz = [];
for i = 1:height(C1) % Loop for each sample
    for j = 1:3 % Loop for each direction
        co = [C1(i,j),C2(i,j),C3(i,j),C4(i,j),C5(i,j)]; % Measured wet conductivities
        cdry = Cdry(i); % Measured dry conductivity
        
        f = @(x,Cf)Cf./x(1).*(1-t+x(1).*(x(2)./Cf)+1/2.*(t-x(2)./Cf).*(1-(x(2)./Cf)./t+sqrt((1-(x(2)./Cf)/t).^2+4.*x(1).*(x(2)./Cf)./t))) + cdry; % RG model
        logf = @(x,Cf)log(Cf./x(1).*(1-t+x(1).*(x(2)./Cf)+1/2.*(t-x(2)./Cf).*(1-(x(2)./Cf)./t+sqrt((1-(x(2)./Cf)/t).^2+4.*x(1).*(x(2)./Cf)./t)))) + cdry; % Log RG model
        
        x0 = [Fi(i),0]; % Initial points

        x(j,:) = lsqcurvefit(logf,x0,Cf,log(co),lb,ub,options); % Least-square curve fitting
        
        co_fit = f(x(j,:),Cf); % Forward solutions
        
        R_sq(i,j) = 1 - sum((log10(co) - log10(co_fit)).^2)/sum((log10(co) - mean(log10(co))).^2); % Coefficient of determination (R^2)
        
        % If R^2 <0.9, recalculate from the determined values
        if R_sq(i) < 0.9
            for k = 1:10
                x0 = x(j,:);
                x(j,:) = lsqcurvefit(logf,x0,Cf,log(co),lb,ub,options);
                co_fit = f(x(j,:),Cf);
            end
            R_sq(i,j) = 1 - sum((log10(co) - log10(co_fit)).^2)/sum((log10(co) - mean(log10(co))).^2);    
        end
    end
    Xx = [Xx;x(1,:)]; % Determined parameters (F,Cs) for X direction
    Xy = [Xy;x(2,:)]; % Determined parameters (F,Cs) for Y direction
    Xz = [Xz;x(3,:)]; % Determined parameters (F,Cs) for Z direction
end
